// Copyright (C) 2015 Torbjorn Sjostrand.
// PYTHIA is licenced under the GNU GPL version 2, see COPYING for details.
// Please respect the MCnet Guidelines, see GUIDELINES for details.

// Example for Les Houches 2015
// Make jets using FastJet and make simplified event record using only leptons and jets

#include "Pythia8/Pythia.h"
#include "fastjet/PseudoJet.hh"
#include "fastjet/ClusterSequence.hh"

using namespace Pythia8;

int main() {

  // Generator. Process selection. LHC initialization. Histogram.
  Pythia pythia;
  pythia.readFile("main105.cmnd");
  pythia.init();

  // Fastjet analysis - select algorithm and parameters
  double Rparam = 0.7;
  fastjet::Strategy               strategy = fastjet::Best;
  fastjet::RecombinationScheme    recombScheme = fastjet::E_scheme;
  fastjet::JetDefinition         *jetDef = NULL;
  jetDef = new fastjet::JetDefinition(fastjet::antikt_algorithm, Rparam,
                                      recombScheme, strategy);

  // Fastjet input
  std::vector <fastjet::PseudoJet> fjInputs;

  // Histograms
  Hist mult("Jet multiplicity", 20, -0.5, 19.5);
  Hist j1pt("hardest jet pT", 100, 0.0, 1000.0);


  // Simplified event
  Event simple;
  simple.init("Simplified event", &pythia.particleData);

  // Event info
  int nEvent = pythia.mode("Main:numberOfEvents");
  int nShowEvent = pythia.mode("Next:numberShowProcess");

  // Begin event loop. Generate event. Skip if error.
  for (int iEvent = 0; ; ++iEvent) {
    if (nEvent > 0 && iEvent > nEvent) break;
    if (!pythia.next()) {
      // If failure because reached end of file then exit event loop.
      if (pythia.info.atEndOfFile()) break;
      continue;
    }
    // Reset simple event record.
    simple.reset();

    // The first four entries decribe initial state
    for (int iEv = 1; iEv < 5; iEv++){
      simple.append(pythia.event[iEv]);
    }

    // Make jets; 
    fjInputs.clear();
    
    // Loop over event record to decide what to pass to FastJet
    for (int i = 0; i < pythia.event.size(); ++i) {
      // Final state only
      if (!pythia.event[i].isFinal()) continue;

      // No neutrinos; add those from W/Z to simple event
      if (pythia.event[i].idAbs() == 12 || pythia.event[i].idAbs() == 14 ||
          pythia.event[i].idAbs() == 16) {
      	int mother = i;
      	bool fromV = false;
      	do {
      	  mother = pythia.event[mother].mother1();
      	  if (abs(pythia.event[mother].id()) == 24 || abs(pythia.event[mother].id()) == 23)
	    {fromV = true; break;}
      	} while (mother > 0);
	
      	if (fromV) simple.append(pythia.event[i]);
	continue;
      }

      // No charged leptons from W/Z-decay; add to simple event instead
      if (pythia.event[i].idAbs() == 11 || pythia.event[i].idAbs() == 13){
	
      	int mother = i;
      	bool fromV = false;
      	do {
      	  mother = pythia.event[mother].mother1();
      	  if (abs(pythia.event[mother].id()) == 24 || abs(pythia.event[mother].id()) == 23)
	    {fromV = true; break;}
      	} while (mother > 0);

      	if (fromV) {
	  simple.append(pythia.event[i]);
      	  continue;
      	}
      }
      
      // Only |eta| < 3.6
      if (fabs(pythia.event[i].eta()) > 3.6) continue;

      // Store as input to Fastjet
      fjInputs.push_back( fastjet::PseudoJet( pythia.event[i].px(),
        pythia.event[i].py(), pythia.event[i].pz(), pythia.event[i].e() ) );
    }

    if (fjInputs.size() == 0) {
      cout << "Error: event with no final state particles" << endl;
      continue;
    }

    // Run Fastjet algorithm
    vector <fastjet::PseudoJet> inclusiveJets, sortedJets;
    fastjet::ClusterSequence clustSeq(fjInputs, *jetDef);
    // Extract inclusive jets sorted by pT (note minimum pT of 20.0 GeV)
    inclusiveJets = clustSeq.inclusive_jets(20.0);
    sortedJets    = sorted_by_pt(inclusiveJets);

    for (unsigned long iJet = 0; iJet < sortedJets.size(); iJet++){
      // Push jets int simplified recore; user defined status 201 => hard jet
      simple.append(21, 201, 3, 4, 0, 0, 0, 0,
		    sortedJets[iJet].px(),sortedJets[iJet].py(),
		    sortedJets[iJet].pz(),sortedJets[iJet].e(),
		    sortedJets[iJet].m());
    }
    
    // Find number of all final charged particles and fill histogram.
    mult.fill( inclusiveJets.size() );
    j1pt.fill( sortedJets[0].pt());
    if (iEvent < nShowEvent) {
      simple.list();
    }
  // End of event loop. Statistics. Histogram.
  }
  pythia.stat();
  cout << mult << j1pt;

  // Done.
  return 0;
}
